//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 1/28/2012.
//


#include "TKLock.h"
#include "TKContainer.h"
#include "TKUICell.h"
#include "DNTimeKeeper.h"
#include "DNServerHTTP.h"
#include "DNServerSerialPort.h"
#include "TKContainer.h"
#include "DNContainerBuilder.h"
#include "DNEngine.h"
#include "DNAlert.h"
#include "DNXML.h"
#include "DNXMLElement.h"
#include "DNGlobal.h"
#include "DNUtils.h"
#include "TKLog.h"

#include <stdlib.h>
#include <sstream>
#include <iostream>

DNEngine::DNEngine(const char *contentPath) :
    mContainer(NULL),mPortNumber(9080),mHTTPServer(NULL),mSerialServer(NULL),mValid(false), mDoTickThread(NULL)
{
    mTimeKeeper = new DNTimeKeeper();
    mContainer = TKContainer::createContainer();
    mUIPath = "/ui/index.html";

    dnGlobal()->updateRunningStatus(DNGlobal::STOPPED);

    std::string basePath(contentPath);
    std::string containerRoot = basePath;
    containerRoot.append("/Container");
    
    bool succeeded = false;
    succeeded = parseSettingFile(contentPath);
    if (!succeeded || !dnGlobal()->isErrorStatusNormal())
    {
        if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
        {
            dnGlobal()->setMessage1("property.xml parse error");
            dnGlobal()->setMessage2("failed to parse setting file /property.xml");
        }
        else
        {
            dnGlobal()->setMessage1("property.xml parse error");
        }
        return;
    }

    std::string dataStorePath = containerRoot;
    dataStorePath.append("/data.db");

    succeeded = mContainer->setDataStore(dataStorePath.c_str());
    if (!succeeded)
    {
        if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
        {
            dnGlobal()->setMessage1("Initialization failed");
            dnGlobal()->setMessage2("Failed to the setup data store");
        }
        else
        {
            dnGlobal()->setMessage1("Initialization failed");
        }
        return;
    }

    succeeded = parseContainerFile(containerRoot.c_str());
    if (!succeeded)
    {
        if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
        {
            dnGlobal()->setMessage1("Initialization failed");
            dnGlobal()->setMessage2("Failed to parse container file");
        }
        else
        {
            dnGlobal()->setMessage1("Initialization failed");
        }
        return;
    }

    mValid = true;
}

DNEngine::~DNEngine()
{
    mValid = false;
    
    if (mContainer)
    {
        delete mContainer;
        mContainer = NULL;
    }
        
    if (mHTTPServer)
    {
        delete mHTTPServer;
        mHTTPServer = NULL;
    }
    
    if (mTimeKeeper)
    {
        delete mTimeKeeper;
        mTimeKeeper = NULL;
    }
}


bool DNEngine::parseSettingFile(const char *contentRoot)
{
    bool valid = false;

    DNXML *xml = DNXML::createXMLFromFile(contentRoot, "property.xml");
    if (xml)
    {
        valid = true;
        DNXMLElement *element = xml->getRoot();
        if (!element)
        {
            valid = false;
            std::string message = "Failed to load property.xml file";
            TKLog::printf("ERROR wile loading property.xml file.\n%s\n",message.c_str());
            if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
            {
                dnGlobal()->setMessage1("Initialization failed");
                dnGlobal()->setMessage2(message);
            }
            else
            {
                dnGlobal()->setMessage1("Initialization failed");
            }
        }

        if (valid && element->name != "dennco")
        {
            valid = false;
            std::string message = "First element of property.xml should be <dennco>";
            TKLog::printf("ERROR wile parsing property.xml file.\n%s\n",message.c_str());
            if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
            {
                dnGlobal()->setMessage1("Initialization failed");
                dnGlobal()->setMessage2(message);
            }
            else
            {
                dnGlobal()->setMessage1("Initialization failed");
            }
        }

        if (valid)
        {
            DNXMLElement *e = element->inner;
            while(e)
            {
                if (e->name == "TickIntervalSec")
                {
                    std::istringstream is(e->text);
                    float t = 0.0;
                    is >> t;
                    if (t>0)
                    {
                        setTickIntervalSec(t);
                    }
                    else
                    {
                        valid = false;
                        std::string message = "Error in property.xml. TickIntervalSec is not configured properly.";
                        TKLog::printf("ERROR wile parsing property.xml file.\n%s\n",message.c_str());
                        if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
                        {
                            dnGlobal()->setMessage1("Initialization failed");
                            dnGlobal()->setMessage2(message);
                        }
                        else
                        {
                            dnGlobal()->setMessage1("Initialization failed");
                        }
                    }
                }
                else if (e->name == "UIPath")
                {
                    mUIPath = e->text;
                }
                else if (e->name == "EnableHTTPServer")
                {

                }
                else if (e->name == "EnableSerialServer")
                {

                }
                e = e->next;
            }
        }

        delete xml;
    }
    return valid;
}

bool DNEngine::parseContainerFile(const char *containerRoot)
{
    DNContainerBuilder builder(mContainer);
    return builder.buildContainerFromXHTML(containerRoot);
}


bool DNEngine::startHTTPServer(int portNumber)
{
    if (mHTTPServer && mHTTPServer->isRunning())
    {
        mHTTPServer->stop();
        delete mHTTPServer;
        mHTTPServer = NULL;
    }
    mHTTPServer = new DNServerHTTP(this);
    if (mHTTPServer)
    {
        mHTTPServer->setPortNumber(portNumber);
        mHTTPServer->start();
    }
    return true;
}

void DNEngine::stopHTTPServer()
{
    if (mHTTPServer && mHTTPServer->isRunning())
    {
        mHTTPServer->stop();
        delete mHTTPServer;
        mHTTPServer = NULL;
    }    
}

bool DNEngine::startSerialServer()
{
    if (mSerialServer && mSerialServer->isRunning())
    {
        mSerialServer->stop();
        delete mSerialServer;
        mSerialServer = NULL;
    }
    mSerialServer = new DNServerSerialPort(this);
    if (mSerialServer)
    {
        if (mSerialServer->setup())
        {
            mSerialServer->start();
            return true;
        }
        else
        {
            return false;
        }
    }
    return false;
}

void DNEngine::stopSerialServer()
{
    if (mSerialServer && mSerialServer->isRunning())
    {
        mSerialServer->stop();
        delete mSerialServer;
        mSerialServer = NULL;
    }
}

void DNEngine::setTickIntervalSec(float interval)
{
    if (mTimeKeeper)
        mTimeKeeper->setIntevalSec(interval);
}

bool DNEngine::startEngine()
{
    if (!mDoTickThread)
        mDoTickThread = DNThread::createThread(DNEngine::doTickThread, this);

    return mDoTickThread->start();
}

bool DNEngine::stopEngine()
{
    bool r = false;
    dnGlobal()->updateRunningStatus(DNGlobal::STOPPING);
    if (mDoTickThread)
    {
        r = mDoTickThread->waitForExit(5000);
        delete mDoTickThread;
        mDoTickThread = NULL;
    }
    if (mContainer)
    {
        mContainer->doDestroy();
        mContainer->releaseDataStore();
    }

    return r;
}

void DNEngine::doTickThread(void *self)
{
    if (!dnGlobal()->updateRunningStatus(DNGlobal::RUNNIING))
    {
        return;
    }

    DNEngine *engine = (DNEngine*)self;
    
    while(dnGlobal()->getRunningStatus() == DNGlobal::RUNNIING && engine->isValid())
    {
        engine->mContainer->doTick(engine->mTimeKeeper->getTickTime());
        engine->mTimeKeeper->sleepUntilNextInterval();
    }

    dnGlobal()->updateRunningStatus(DNGlobal::STOPPED);
}

float DNEngine::doClientGetRequest(const char* path)
{
    std::string fqn = getFQNString("/",path);
    TKUICell *cell = (TKUICell*)mContainer->getInterfaceCell(fqn);
    float result = 0;
    if (cell)
    {
        result = cell->getValue();
    }
    return result;
}

bool DNEngine::doClientSetRequest(const char* path, const char* value)
{
    return doClientSetRequest(path, atof(value));
}

bool DNEngine::doClientSetRequest(const char* path, float value)
{
    std::string fqn = getFQNString("/",path);
    TKUICell *cell = (TKUICell*)mContainer->getInterfaceCell(fqn);
    bool result = false;
    if (cell)
    {
        cell->setValue(value);
        result = true;
    }
    return result;
}

std::string DNEngine::getContentPath()
{
    if (mContainer)
        return mContainer->getContentPath();
    else
        return "";
}

std::string DNEngine::getUIPath()
{
    return mUIPath;
}
