//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 2/22/2012.
//
#include "dnqscellcode.h"

#include "DNUtils.h"
#include "TKLog.h"
#include "TKContainer.h"
#include "dnqscontainer.h"
#include "dnqscellbase.h"
#include "dnqscellcodeinstance.h"

DNQSCellCode::DNQSCellCode(std::string name, std::string cellapi, DNQSContainer *container, std::string code):
    TKCellCode(name,cellapi), mCellContainer(container)
{
    std::string jname = getJSEscapeString(name.c_str());
    QString stmt = QString::fromStdString(parseCellCodeForScriptEngine(jname, code));

    QScriptEngine *scriptEngine = mCellContainer->getScriptEngine();
    QScriptValue  scriptGlobalObject = mCellContainer->getScriptGlobalObject();
    scriptEngine->evaluate(QScriptProgram(stmt));
    mCellCodeConstructor = scriptGlobalObject.property(QString::fromStdString(jname), QScriptValue::ResolveLocal);
    if (mCellCodeConstructor.isNull() || mCellCodeConstructor.isUndefined() || ! mCellCodeConstructor.isValid())
    {
        TKLog::printf("DNQSCellCode ERROR creating:%s", name.c_str());
    }
}

DNQSCellCode::~DNQSCellCode()
{

}
TKCellCodeInstance* DNQSCellCode::createCellCodeInstance(TKCell *owner, const void *data)
{
    if (!mCellContainer)
    {
        return NULL;
    }
    if (mCellCodeConstructor.isNull() || mCellCodeConstructor.isUndefined() || ! mCellCodeConstructor.isValid())
    {
        TKLog::printf("DNQSCellCode::createCellCodeInstance ERROR. %s", getName().c_str());
        return NULL;
    }

    TKCellCodeInstance *result = NULL;

    QScriptValueList args;
    args << ((DNQSCellBase*)owner)->getCellAPI();
    QScriptValue instance = mCellCodeConstructor.construct(args);

    if (instance.isValid())
    {
        QScriptEngine *scriptEngine = mCellContainer->getScriptEngine();

        //
        QScriptContext *context = scriptEngine->pushContext();
        context->setThisObject(instance);
        context->setActivationObject(instance);
        QString customScriptStr((const char*)data);
        QScriptProgram customScript(customScriptStr);
        scriptEngine->evaluate(customScript);

        scriptEngine->popContext();

        result = new DNQSCellCodeInstance(owner, instance);

    }

    return result;
}
