//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 12/11/2011.
//

#include "TKCell.h"
#include "TKReceptor.h"
#include "TKContainer.h"
#include "TKCellCode.h"
#include "TKAxon.h"
#include "TKCellCodeInstance.h"
#include "TKLog.h"
#include <string>

TKCell::TKCell(TKContainer *container, std::string location, std::string name, bool canIntarface)
: mName(name), mLocation(location), mContainer(container), mAxon(NULL), mCellCodeInstance(NULL), mCanInterface(canIntarface)
{
    mAxon = new TKAxon(this);
}

TKCell::~TKCell()
{
    if (mCellCodeInstance)
    {
        delete mCellCodeInstance;
        mCellCodeInstance = NULL;
    }

	for ( TKReceptorMap::iterator it = mReceptors.begin(); it != mReceptors.end(); ++it ) {
		delete it->second;
	}
    mReceptors.clear();

    if (mAxon)
    {
        delete mAxon;
        mAxon = NULL;
    }
}

float TKCell::getAxonValue()
{
    if (!mAxon)
        return 0.0;
    return mAxon->getValue();
}

void  TKCell::setAxonValue(float value)
{
    if (!mAxon)
        return;
    mAxon->setValue(value);
}

bool TKCell::setCellCode(TKCellCode *code, const void *data)
{
    mCellCodeInstance = code->createCellCodeInstance(this, data);
    if (mCellCodeInstance)
        return doInit();
    else
        return false;
}

bool TKCell::connectTo(std::string connectionName, TKCell *targetCell)
{
    TKReceptor *receptor = targetCell->createReceptor(connectionName);    
    TKAxonTerminal *terminal = mAxon->addTerminal();
    receptor->setTarget(terminal);
    
    return true;
}

TKReceptor* TKCell::createReceptor(std::string name)
{
    TKReceptor *receptor;
    TKReceptorMap::iterator it = mReceptors.find(name);
    if (it != mReceptors.end())
    {
        receptor = it->second;
    }
    else
    {
        receptor = new TKReceptor(this);
        if (receptor)
        {
            mReceptors.insert( TKReceptorMap::value_type( name, receptor ) );
        }
    }
    
    return receptor;
}

