//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 12/11/2011.
//

#ifndef __INCLUDE_TKCONTAINER__
#define __INCLUDE_TKCONTAINER__

#include "TKLock.h"

#include <string>
#include <map>

class TKCell;
class TKCellCode;
class DNStorage;

typedef std::map<std::string, TKCell*> TKCellMap;
typedef std::map<std::string, TKCellCode*> TKCellCodeMap;

class TKContainer
{
public:
    static TKContainer* createContainer();

    virtual ~TKContainer();

    const TKCellMap* getCells() { return &mCells; } 
    const TKCellCodeMap* getCellCodes() { return &mCellCodes; }

    void        setContentPath(std::string contentPath) { mContentPath = contentPath; }
    std::string getContentPath() { return mContentPath; }
    bool        setDataStore(std::string storagePath);
    DNStorage*  getDataStore() { return mStorage; }
    bool        releaseDataStore();
    TKCell*     getCell(std::string theFQNName);
    TKCell*     getInterfaceCell(std::string theFQNName);
    TKCellCode* getCellCode(std::string theCellCodeName);

    bool        doTick(float time);
    bool        doDestroy();

    virtual TKCell*     createCellWithoutCellCodeClass(std::string theLocation, std::string theName, std::string type, std::string customScript) = 0;
    virtual TKCell*     createCellWithCellCodeClass(std::string theLocation, std::string theName, TKCellCode *cellCode, std::string customScript) = 0;
    virtual TKCellCode* createCellCode(std::string theName, std::string theAPIType, std::string code) = 0;
    	
    virtual void setValue(std::string key, float value) = 0;
    inline virtual float getValue(std::string key) = 0;

protected:
    TKContainer() : mStorage(NULL){}
    TKCellMap       mCells;
    TKCellMap       mInterfaceCells;
    TKCellCodeMap   mCellCodes;
    std::string     mContentPath;
    DNStorage       *mStorage;

private:
    TKLock          mLock;
};
#endif
