//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on Apr/15/2012.
//
#include "qtdnserialportimpl.h"

#include "DNServerSerialPort.h"
#include "DNThread.h"
#include "DNUtils.h"
#include <QThread>

class SleeperThread : public QThread
{
public:
    static void msleep(unsigned long msecs)
    {
        QThread::msleep(msecs);
    }
};

DNSerialPortImpl * DNSerialPortImpl::create(DNServerSerialPort *server)
{
    return new QtDNSerialPortImpl(server);
}


QtDNSerialPortImpl::QtDNSerialPortImpl(DNServerSerialPort *server) :
    DNSerialPortImpl(server), mListnerThread(NULL), mHandlerThread(NULL), mPort(NULL), mStopping(false)
{

}

QtDNSerialPortImpl::~QtDNSerialPortImpl()
{
    stop();
    if (mListnerThread)
    {
        delete mListnerThread;
        mListnerThread = NULL;
    }
    if (mHandlerThread)
    {
        delete mHandlerThread;
        mHandlerThread = NULL;
    }
    if (mPort)
    {
        delete mPort;
        mPort = NULL;
    }
}

bool QtDNSerialPortImpl::setup()
{
    //TODO
    return false;
}

bool QtDNSerialPortImpl::isRunning()
{
    //TODO
    return false;
}

void QtDNSerialPortImpl::start()
{
    stop();
    if (!mListnerThread)
        mListnerThread = DNThread::createThread(QtDNSerialPortImpl::requestListenerThreadBody, this);

    if (!mHandlerThread)
        mHandlerThread = DNThread::createThread(QtDNSerialPortImpl::requestHandelrThreadBody, this);

    mListnerThread->start();
    mHandlerThread->start();
}

void QtDNSerialPortImpl::stop()
{

}

//static
void QtDNSerialPortImpl::requestListenerThreadBody(void *_impl)
{
    if (!_impl)
        return;

    QtDNSerialPortImpl *impl = (QtDNSerialPortImpl*)_impl;
    char buffer[1024];
    while(!impl->mStopping)
    {
        if ((impl->mPort->bytesAvailable() > 0) ||  impl->mPort->waitForReadyRead(100))
        {
            int len = impl->mPort->readLine(buffer, sizeof(buffer));
            if (len != -1)
            {
                impl->mLock.lock();
                impl->mRequestQueue.push_back(QString::fromLocal8Bit(buffer));
                if (impl->mQueueSemaphore.available()==0)
                {
                   impl->mQueueSemaphore.release(1);
                }
                impl->mLock.unlock();
            }
        }
        if (!impl->mPort->isOpen())
        {
            SleeperThread::msleep(300);
        }
    }
}

//static
void QtDNSerialPortImpl::requestHandelrThreadBody(void *_impl)
{
    if (!_impl)
        return;

    QtDNSerialPortImpl *impl = (QtDNSerialPortImpl*)_impl;
    while(!impl->mStopping)
    {
        impl->mQueueSemaphore.tryAcquire(1,100);
        impl->mLock.lock();
        if (!impl->mRequestQueue.isEmpty())
        {
            QString line = impl->mRequestQueue.dequeue();
            impl->mLock.unlock();

            //TODO

        }
        else
        {
            impl->mLock.unlock();
        }

    }


}

