//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 2/25/2012.
//
#include "qtdntimekeeperimpl.h"

#include <QThread>
#include <QDateTime>

class SleeperThread : public QThread
{
public:
    static void msleep(unsigned long msecs)
    {
        QThread::msleep(msecs);
    }
};

DNTimeKeeperImpl *DNTimeKeeperImpl::create()
{
    return new QtDNTimeKeeperImpl;
}

QtDNTimeKeeperImpl::QtDNTimeKeeperImpl() : mIntervalTime(1000),mAvgActualInterval(1000),mAdjustTime(0),mPrevTime(0),mStartTime(0),mTickTime(0)
{
}

QtDNTimeKeeperImpl::~QtDNTimeKeeperImpl()
{
}

bool  QtDNTimeKeeperImpl::setIntevalSec(float time)
{
    mIntervalTime = time * 1000;
    mAvgActualInterval = time * 1000;
    mAdjustTime = 0.0;
    return true;
}

bool  QtDNTimeKeeperImpl::sleepUntilNextInterval()
{
    if (mStartTime == 0)
    {
        start();
    }
    qint64 currentTime = QDateTime::currentMSecsSinceEpoch();
    qint64 dt = currentTime - mPrevTime;
    qint64 wt = mIntervalTime + dt + (int)mAdjustTime;

    if (wt < 5)  // wait at least 5ms.
    {
        wt = 5;
    }
    SleeperThread::msleep(wt);
    qint64 t = QDateTime::currentMSecsSinceEpoch();
    mAvgActualInterval = (float)mAvgActualInterval * 0.9 + ((float)(t - mPrevTime) * 0.1);
    mAdjustTime += (mIntervalTime - mAvgActualInterval);
    mPrevTime = t;

    mTickTime = (mPrevTime - mStartTime)/1000.0;

    return true;
}

float QtDNTimeKeeperImpl::getTickTime()
{
    return mTickTime;
}

bool QtDNTimeKeeperImpl::start()
{
    mStartTime = QDateTime::currentMSecsSinceEpoch();
    mPrevTime = mStartTime;
    mAvgActualInterval = mIntervalTime;
    mAdjustTime = 0.0;
    mTickTime = 0.0;
    return true;
}

bool QtDNTimeKeeperImpl::stop()
{
    mStartTime = 0;
    return true;
}
